# -*- coding: utf-8 -*-
# dreamtools-dreamgeeker/tracking_manager.py
import asyncio
import inspect
import logging
import logging.config as log_config

from . import file_manager
from . import toolbox
from .config_manager import ConfigController
from .controller_manager import ControllerEngine
from .exception_manager import ExceptionManager, Reponce


class TrackingManager:
    tracker: logging.Logger
    system_tracker: logging.Logger = None
    LOG_TRACKED = ''

    @staticmethod
    def initialisation(path_cfg='.cfg/logcfg', logger='PRODUCTION', **params):
        """Initialisation du gestionnaire de log à partir de la configuration enregistré

        .. warning::
            La configuration doit être configurer dans le fichier <PROJECT_DIR>/cdg/.log.yml

        :Exemple:cls
            >>> TrackingManager.initialisation('/projet/dir', path_cfg='.cfg/log.yml', logger='debug')
        """
        if not ControllerEngine.PROJECT_DIR:
            toolbox.print_err('Initialisation manquante')
            return False

        try:

            log_path = file_manager.path_build(ControllerEngine.PROJECT_DIR, path_cfg)
            file_manager.makedirs(log_path)
            cfg = ConfigController.loading(log_path)

            for f_handler in ['file_info', 'file_system']:
                f_p = cfg['handlers'][f_handler]['filename']
                if params.get('project_name'):
                    path_dir = f'logs/{params.get('project_name')}/'
                    cfg['handlers'][f_handler]['filename'] = f_p.replace('logs/', path_dir)
                    file_manager.makedirs(path_dir)

            log_config.dictConfig(cfg)
            TrackingManager.tracker = logging.getLogger(logger)
            TrackingManager.system_tracker = logging.getLogger("system")

        except Exception as e:
            toolbox.print_err(e, ': ', 'Error in Logging Configuration. Using default configs')
            logging.basicConfig(level=logging.NOTSET)

    @staticmethod
    def msg_tracking(msg, title, log_level=logging.INFO):
        """ Tracking message

        :param str msg: message à ecrire dans logs
        :param str title: Titre ou référence associé au message
        :param int log_level: LOG LEVEL Niveau de l'alert (DEBUG | INFO | WARN | )

        """

        TrackingManager.tracker.log(log_level, msg, exc_info=True, stack_info=False, stacklevel=2,
                                    extra={'title': title, 'stacklevel': 2})

    @staticmethod
    def alert_tracking(msg, title):
        """ Message d'alerte (WARNING)

        :param str msg: message à ecrire dans logs
        :param str title: Titre ou référence associé au message
        """
        TrackingManager.msg_tracking(msg, title, logging.WARNING)

    @staticmethod
    def info_tracking(msg, title):
        """ Message d'info (INFO)

        :param str msg: message à ecrire dans logs
        :param str title: Titre ou référence associé au message
        """

        TrackingManager.msg_tracking(msg, title, logging.INFO)

    @staticmethod
    def error_tracking(msg, title):
        """ Message d'error (ERROR)
        :param str msg: message à ecrire dans logs
        :param str title: Titre ou référence associé au message
        """
        TrackingManager.msg_tracking(msg, title, logging.ERROR)

    @staticmethod
    def critical_tracking(msg, title):
        """ Message dcritique (CRITIQUE)
        :param str msg: message à ecrire dans logs
        :param str title: Titre ou référence associé au message
        """
        TrackingManager.msg_tracking(msg, title, logging.CRITICAL)

    @staticmethod
    def flag(trace):
        """ Permet de pointer la dernier action

        :param str trace: Action à enregistrer
        """
        TrackingManager.LOG_TRACKED = trace

    @staticmethod
    def exception_tracking(ex, title: str, status=500) -> ExceptionManager:
        """
        Récupération et traitement des exceptions

        :param status: 
        :param ex: Exception
        :param str title: Information
        """

        try:
            status = getattr(ex, 'status', status) or status
            if TrackingManager.LOG_TRACKED != '':
                if TrackingManager.system_tracker:
                    TrackingManager.system_tracking('## ' + TrackingManager.LOG_TRACKED + ' ##', title, logging.INFO)
                    TrackingManager.msg_tracking('## ' + TrackingManager.LOG_TRACKED + ' ##', title, logging.INFO)
                TrackingManager.LOG_TRACKED = ''

            if isinstance(ex, ExceptionManager):
                TrackingManager.error_tracking(ex.message, ex.title)
                return ex
            else:
                TrackingManager.error_tracking(str(ex), title)
                return ExceptionManager(str(ex), title)

        except Exception as sex:
            toolbox.print_err('*****************  Erreur système *****************: \n', str(sex))
            toolbox.print_err('** Erreur interceptée : ', str(ex))
            return ExceptionManager(str(ex), status=status)

    @staticmethod
    def fntracker(fn, action, *args, **kwargs):
        """ Execution "securisé" d'une fonction avec gestions des erreurs
        
        :param fn: fonction a executer
        :param action: Titre de l'execution pour tracabilité
        :param args: argument de la fonction
        :param kwargs: parametres supplementaire (status par defaut en cas de reussite)
        :rtype: CReponder

        :Exemple:
            >>> r = TrackingManager.fntracker(fn, 'Test de convertion int', 'j')
            >>> r.response
            {'message': "invalid literal for int() with base 10: 'j'", 'data': None, 'status_code': 500}
            >>> r = TrackingManager.fntracker(fn, 'Test de convertion int', '589321')
            >>> r.response
            {'message': None, 'data': 589321, 'status_code': 200}
        """

        try:
            TrackingManager.flag('{}'.format(action))
            if inspect.iscoroutinefunction(fn):
                r = asyncio.run(fn(*args, **kwargs))
            else:
                r = fn(*args, **kwargs)

            return r if isinstance(r, Reponce) else Reponce("Action réussi", action, 200, r)

        except Exception as ex:
            return TrackingManager.exception_tracking(ex, action)

    @staticmethod
    def system_tracking(msg, title, log_level=logging.WARNING):
        """ Tracking message
        :param str msg: message à ecrire dans logs
        :param str title: Titre ou référence associé au message
        :param int log_level: LOG LEVEL Niveau de l'alert (DEBUG | INFO | WARN | )
        """
        TrackingManager.system_tracker.log(log_level, msg, extra={'title': title})



import re

from cerberus import Validator, TypeDefinition
from cerberus.validator import schema_registry

from . import toolbox

# --- Types personnalisés ---
custom_types = {'phone': TypeDefinition('phone', (str,), ()), 'password': TypeDefinition('password', (str,), ()), }


# --- Fonction de nettoyage ---
def normalize_phone(value):
    if not isinstance(value, str):
        return value or ''
    value = value.strip()
    if value.startswith('+'):
        value = '+' + re.sub(r'[^\d]', '', value[1:])
    else:
        value = re.sub(r'[^\d]', '', value)

    match = re.match(r'(00|\+)?(\d+)(\d{3})(\d{3})(\d{3})', value)
    if match:
        prefix = ''
        groups = match.groups()
        if groups[0] == '+':
            prefix = '+'
            groups = groups[1:]
        elif groups[0] is None:
            groups = groups[1:]
        return prefix + ' '.join(groups)
    return value or ''


class DreamRegistry:
    email = {'type': 'string', 'is_email': True, 'coerce': lambda v: '' if v is None else v, 'empty': True}
    password = {'type': 'string', 'is_password': True, 'empty': False}
    url = {'type': 'string', 'is_url': True, 'coerce': lambda v: '' if v is None else v, 'empty': True}
    varchar = {'type': 'string', 'coerce': toolbox.clean_space, 'maxlength': 255, 'empty': True, }
    referenco = {'type': 'string', 'coerce': toolbox.clean_allspace, 'maxlength': 15, 'empty': True, }
    referenco_short = {'type': 'string', 'coerce': toolbox.clean_allspace, 'maxlength': 4, 'empty': True, }
    phone = {'type': 'string', 'coerce': normalize_phone, 'is_phone': True, 'empty': True, }
    is_digit = {'type': 'integer', 'coerce': lambda v: int(v) if v and (isinstance(v, int) or v.isdigit()) else v}


# --- Schémas génériques enregistrés ---
schema_registry.add('email_schema', {'email': {'type': 'string', 'regex': toolbox.RGX_EMAIL}})
schema_registry.add('email_schema', {'email': DreamRegistry.email})
schema_registry.add('password_schema', {'password': DreamRegistry.password})
schema_registry.add('phone_number_schema', {'phone': DreamRegistry.phone})
schema_registry.add('url_schema', {'url': DreamRegistry.url})
schema_registry.add('link_schema', {'link': DreamRegistry.url})

schema_registry.add('contact_info_schema',
                    {'email': DreamRegistry.email, 'phone_1': DreamRegistry.phone, 'phone_2': DreamRegistry.phone,
                        'oneof': [{'email': {'required': True}}, {'phone_1': {'required': True}},
                                  {'phone_2': {'required': True}}]})

schema_registry.add('authentication_schema', {'email': DreamRegistry.email | {'required': True},
    'password': DreamRegistry.password | {'required': True}})

schema_registry.add('signin_schema', {'email': DreamRegistry.email | {'required': True},
    'password': DreamRegistry.password | {'required': True},
    'confirmpassword': DreamRegistry.password | {'required': True}})


# --- Validator personnalisé ---
class DreamValidator(Validator):
    types_mapping = Validator.types_mapping.copy()
    types_mapping.update(custom_types)

    def _validate_is_email(self, is_email, field, value):
        """ {'type': 'boolean'} """
        if is_email and not toolbox.is_valid_email(value):
            self._error(field, "L'adresse email est invalide.")

    def _validate_is_url(self, is_url, field, value):
        """ {'type': 'boolean'} """
        if is_url and not toolbox.is_valid_url(value):
            self._error(field, "L'URL est invalide.")

    def _validate_is_password(self, is_password, field, value):
        """ {'type': 'boolean'} """
        if is_password and not toolbox.is_valid_password(value):
            self._error(field, "Le mot de passe est invalide.")

    def _validate_is_phone(self, is_phone, field, value):
        """ {'type': 'boolean'} """
        if is_phone and not toolbox.is_valid_phone(value):
            self._error(field, "Le numéro de téléphone est invalide.")


# --- Fonction générique de validation ---
def validate_data(data: dict, schema_name: str = None, schema: dict = None) -> tuple[bool, dict]:
    """
    Valide les données selon un schéma donné ou un nom de schéma enregistré.

    :param data: Données à valider.
    :param schema_name: Nom d’un schéma enregistré dans schema_registry.
    :param schema: Schéma inline personnalisé.
    :return: Tuple (est_valide, erreurs).

    :Exemple:
    data = { 'email': 'test@exemple.com', 'phone_number': normalize_phone('06 90 12 34 56')}
    ok, errors = validate_data(data, schema_name='contact_info_schema')

    if ok:
        print("✓ Données valides")
    else:
        print("✗ Erreurs :", errors)
    """
    if schema_name:
        schema = schema_registry.get(schema_name)
        if not schema:
            raise ValueError(f"Schéma '{schema_name}' non trouvé dans le registre.")

    if not schema:
        raise ValueError("Aucun schéma fourni pour la validation.")

    validator = DreamValidator(schema)
    is_valid = validator.validate(data)
    return is_valid, validator.errors



class Reponce:
    status:int
    detail = None
    message:str
    title:str

    def __init__(self, message:str='Opération réussie', title:str='', status:int=200, *args, **kwargs):
        self.status = status
        self.detail = args or kwargs
        self.message = message
        self.title = title


    @property
    def succeed(self):
        """ Renvoie le status de la réponse

        :rtype: bool
        """
        return 200 <= self.status < 300



class ExceptionManager(Exception):
    """
    Exception personnalisée avec suivi automatique via le tracker.

    :param message: Message lisible destiné à l’utilisateur.
    :param status: Code HTTP de l’erreur.
    :param title: Titre bref, destiné aux logs et à l’interface.
    """

    def __init__(self, message="Une erreur interne est survenue.", title='ERROR SYSTEM', status=500, **kwargs):
        super().__init__(message, title, status)

        self.message = message
        self.title = title
        self.status = status
        self.detail = kwargs


    def __str__(self):
        return str(f'[{self.title}] : self.message - {self.status}')

class RequestException(ExceptionManager):
    def __init__(self):
        super().__init__(
            "Le format ou le contenu de la requête n’est pas accepté.",
            "Requête non acceptable",
            406
        )


class AuthException(ExceptionManager):
    def __init__(self):
        super().__init__(
            "Identifiants incorrects. Veuillez vérifier votre login ou mot de passe.",
            "Échec d’authentification",
            401
        )


class OAuthException(ExceptionManager):
    def __init__(self):
        super().__init__(
            "Vous n’avez pas les autorisations nécessaires pour accéder à cette ressource.",
            "Accès interdit",
            403
        )


class OAuthElapsedException(ExceptionManager):
    def __init__(self):
        super().__init__(
            "Le délai d’authentification a expiré. Veuillez recommencer.",
            "Délai dépassé",
            408
        )


class UExistException(ExceptionManager):
    def __init__(self):
        super().__init__(
            "Ce compte existe déjà. Veuillez utiliser une autre adresse ou réinitialiser votre mot de passe.",
            "Conflit d'identité",
            409
        )


class AccountException(ExceptionManager):
    def __init__(self):
        super().__init__(
            "Aucun compte correspondant n’a été trouvé.",
            "Compte introuvable",
            404
        )


class RessourceException(ExceptionManager):
    def __init__(self):
        super().__init__(
            "La ressource demandée est introuvable ou n’existe plus.",
            "Ressource introuvable",
            404
        )


class PageException(ExceptionManager):
    def __init__(self):
        super().__init__(
            "La page que vous cherchez n’existe pas.",
            "Page non trouvée",
            404
        )


class ParamsException(ExceptionManager):
    def __init__(self):
        super().__init__(
            "Les données soumises sont invalides ou incomplètes.",
            "Paramètres incorrects",
            400
        )


class TooManyRequests(ExceptionManager):
    def __init__(self):
        super().__init__(
            "Trop de requêtes ont été envoyées en peu de temps. Merci de patienter avant de réessayer.",
            "Requêtes trop fréquentes",
            429
        )


from argon2.exceptions import VerifyMismatchError
from dreamtools import toolbox
from sanic_ext.extensions.openapi.openapi import document

from .toolbox import ensure_bytes

_all_ = ['CryptoManager']
import base64
import json
import os
import secrets
import time
from base64 import b64encode, b64decode

import base58
from argon2 import PasswordHasher
from cryptography.fernet import Fernet
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives import serialization
from cryptography.hazmat.primitives.asymmetric import x448
from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes
from cryptography.hazmat.primitives.kdf.hkdf import HKDF


class CryptoManager:
    """
    Gère les opérations cryptographiques (génération de clés, chiffrement/déchiffrement, stockage en base).

    - Utilise X448 pour l'échange de clés.
    - AES-GCM pour le chiffrement symétrique.
    - Clés stockées chiffrées dans la base avec fingerprint.
    """
    master_key = None
    ph = PasswordHasher()

    @staticmethod
    def set_master_key(master_key: str | bytes):
        CryptoManager.master_key = ensure_bytes(master_key)

    @staticmethod
    def generate_fingerprint(public_bytes: str|bytes) -> str:
        """Génère un fingerprint SHA-256 pour une clé publique"""
        public_bytes = ensure_bytes(public_bytes)       # on s'assure que c'est du bytes
        fingerprint = hashes.Hash(hashes.SHA256())
        fingerprint.update(public_bytes)
        fingerprint_result = fingerprint.finalize()
        return base58.b58encode(fingerprint_result).decode()

    @staticmethod
    def get_fingerprint(public_key) -> str:
        """Retourne un digest SHA256 tronqué (16 premiers caractères) d'une clé publique."""
        raw_bytes = public_key.public_bytes(
            encoding=serialization.Encoding.Raw,
            format=serialization.PublicFormat.Raw
        )
        digest = hashes.Hash(hashes.SHA256(), backend=default_backend())
        digest.update(raw_bytes)
        full_digest = digest.finalize()
        base58_digest = base58.b58encode(full_digest).decode()
        return base58_digest[:16]

    @staticmethod
    def verify_fingerprint(public_bytes: str|bytes, stored_fingerprint: str) -> bool:
        """Vérifie que l'empreinte générée correspond à celle stockée."""
        fp1 = CryptoManager.generate_fingerprint(public_bytes)
        return fp1 == stored_fingerprint

    @staticmethod
    async def generate_keys()->(bytes, bytes, bytes):
        """Génère une paire de clés X448, stocke en DB et retourne les éléments."""
        private_key = x448.X448PrivateKey.generate()
        private_bytes = private_key.private_bytes(
            encoding=serialization.Encoding.Raw,
            format=serialization.PrivateFormat.Raw,
            encryption_algorithm=serialization.NoEncryption()
        )
        public_key = private_key.public_key()
        public_bytes = public_key.public_bytes(
            encoding=serialization.Encoding.Raw,
            format=serialization.PublicFormat.Raw
        )
        finger_print = CryptoManager.generate_fingerprint(public_bytes)

        return public_bytes, private_bytes, finger_print

    @staticmethod
    def exchange_shared_key(private_key_bytes: bytes, public_key_bytes: bytes):
        """Échange de clés X448 pour générer une clé partagée"""
        private_key_bytes = ensure_bytes(private_key_bytes)
        public_key_bytes = ensure_bytes(public_key_bytes)

        private_key = x448.X448PrivateKey.from_private_bytes(private_key_bytes)
        public_key = x448.X448PublicKey.from_public_bytes(public_key_bytes)
        shared_key = private_key.exchange(public_key)

        return shared_key

    @staticmethod
    def encrypt_data(dc: str|bytes, shared_key, client_secret) -> str:
        # Générer les clés X448 pour l'expéditeur et le récepteur
        iv = os.urandom(12)  # IV pour AES-GCM
        aes_key = CryptoManager.derive_aes_key(shared_key, client_secret)  # Dériver la clé AES à partir de la clé partagée
        encryptor = Cipher(algorithms.AES(aes_key),modes.GCM(iv),backend=default_backend()).encryptor()
        data_bytes = toolbox.ensure_bytes(dc)
        ciphertext = encryptor.update(data_bytes) + encryptor.finalize()
        tag = encryptor.tag

        encryted_data = {
            'ciphertext': CryptoManager.encode_secret(ciphertext),
            'iv': CryptoManager.encode_secret(iv),
            'tag': CryptoManager.encode_secret(tag)
        }
        return json.dumps(encryted_data)

    @staticmethod
    def encrypt_message(dc: dict|str, receiver_public_key, sender_private_key, sender_client_secret)->str:
        # Générer les clés X448 pour l'expéditeur et le récepteur
        if isinstance(dc, dict):
            dc = json.dumps(dc)

        shared_key = CryptoManager.exchange_shared_key(sender_private_key, receiver_public_key)
        # Chiffrer le message
        return CryptoManager.encrypt_data(dc, shared_key, sender_client_secret)

    @staticmethod
    def encrypt_password(password):
        """
        Argon2 est une fonction de hachage de mot de passe lauréate du concours Password Hashing Competition.
        """
        return CryptoManager.ph.hash(password)

    @staticmethod
    def decrypt_encrypted_data(encrypted_data:str, shared_key: str|bytes, secret_key: str|bytes) -> str:
        """Déchiffre les données avec AES-GCM en utilisant la clé partagée dérivée"""
        dc = json.loads(encrypted_data)
        iv = CryptoManager.decode_secret(dc['iv'])
        tag = CryptoManager.decode_secret(dc['tag'])
        ciphertext = CryptoManager.decode_secret(dc['ciphertext'])

        aes_key = CryptoManager.derive_aes_key(shared_key, secret_key)  # Dériver la clé AES à partir de la clé partagée
        cipher_decrypt = Cipher(algorithms.AES(aes_key),modes.GCM(iv, tag),backend=default_backend()).decryptor()
        decypted_message = cipher_decrypt.update(ciphertext) + cipher_decrypt.finalize()

        return toolbox.ensure_string(decypted_message)

    @staticmethod
    def decrypt_encrypted_message(message_crypted, receiver_private_key: bytes, sender_public_key: bytes, sender_client_secret: bytes) -> str:
        """Déchiffre un message JSON en utilisant AES-GCM et l'échange de clés X448"""
        shared_key = CryptoManager.exchange_shared_key(receiver_private_key, sender_public_key)
        return CryptoManager.decrypt_encrypted_data(message_crypted, shared_key, sender_client_secret)

    @staticmethod
    def check_password(password, hasher):
        """
        Vérification du mot de passe
        """
        try:
            return CryptoManager.ph.verify(hasher, password)
        except VerifyMismatchError:
            return False

    @staticmethod
    def generate_bytes_secret(x: int = 24) -> bytes:
        """
        Génère une clé secrète aléatoire sous forme d'octets sécurisés.
    
        :param x: La longueur de la clé en octets. Par défaut, la longueur est de 24 octets.
        :type x: Int
        :return: Une séquence d'octets aléatoires sécurisés.
        :rtype: Bytes
    
        : Example :
    
        >>> CryptoManager.generate_bytes_secret(16)
        b'\x8a\xe4\xb3\xf0\xd5\xc7\xe9\x89\xaa\xa3~\xd1\x92\xc5\xb5'
        """
        return secrets.token_bytes(x)

    @staticmethod
    def generate_hex_secret(x=16) -> str:
        """Génère une clé secrète aléatoire sous forme de chaîne hexadécimale.
    
        :param x: La longueur de la chaîne hexadécimale. Par défaut, la longueur est déterminée automatiquement.
        :type x: Int, optional
        :return: Une chaîne hexadécimale aléatoire.
        :rtype: Str
    
        >>> CryptoManager.generate_hex_secret(16)
        'a1b2c3d4e5f6'
        """
        return secrets.token_hex(x)

    @staticmethod
    def code_unique(texte):
        timestamp = str(int(time.time()))
        texte_timestamp = texte + timestamp
        hasher = hashes.Hash(hashes.SHA1())
        hasher.update(texte_timestamp.encode())
        code_hash = hasher.finalize().hex()
        return code_hash[:8]  # Récupère les 8 premiers caractères du code de hachage

    @staticmethod
    def derive_aes_key(auth_key: str|bytes, salt_key: str|bytes)->bytes:
        """Dérive une clé AES de 256 bits à partir de la clé partagée X448 en utilisant HKDF"""
        # Utiliser HKDF pour dériver une clé AES 256 bits
        auth_key = toolbox.ensure_bytes(auth_key)
        salt_key = toolbox.ensure_bytes(salt_key)

        hkdf = HKDF(
            algorithm=hashes.SHA256(),
            length=32,  # 32 bytes = 256 bits
            salt=salt_key,
            info=b"SharedKeyDerivation",
            backend=default_backend()
        )
        return hkdf.derive(auth_key)

    @staticmethod
    def fernet_engine(auth_key: str|bytes, salt_key: str|bytes)->Fernet:
        """
            Déchiffre les données d'un fichier et les retourne.
    
            Args:
            fs (str): Le chemin du fichier contenant les données chiffrées.
    
            Returns:
            str: Les données déchiffrées.
        """
        key_derived = CryptoManager.derive_aes_key(auth_key, salt_key)
        fernet_key = base64.urlsafe_b64encode(key_derived)
        return Fernet(fernet_key)

    @staticmethod
    def basic_auth_encode(s_id, s_sekret):
        """Generation d'un autorization basic de type ID:SECRET"""
        access_token = CryptoManager.encode_secret(f"{s_id}:{s_sekret}")
        return f"Basic {access_token}"

    @staticmethod
    def basic_auth_decode(access_token):
        """Generation d'un autorization basic de type ID:SECRET"""
        access_token = toolbox.ensure_string(CryptoManager.decode_secret(access_token))
        return access_token.split(":")

    @staticmethod
    def encode_secret(secret_data: bytes | str) -> str:
        secret_data = toolbox.ensure_bytes(secret_data)
        encrypted_data = b64encode(secret_data)
        return toolbox.ensure_string(encrypted_data)

    @staticmethod
    def decode_secret(secret_data: bytes | str) -> bytes:
        secret_data = toolbox.ensure_string(secret_data)
        encrypted_data = b64decode(secret_data)
        return encrypted_data

    @staticmethod
    def encrypt_secret(secret_data, auth_key, salt_key) -> str:
        b_cs = toolbox.ensure_bytes(secret_data)
        engine = CryptoManager.fernet_engine(auth_key, salt_key)
        secret_data = engine.encrypt(b_cs)
        return toolbox.ensure_string(secret_data)

    @staticmethod
    def decrypt_secret(encrypted_secret: bytes | str, auth_key: bytes | str, salt_key: bytes | str) -> str:
        engine = CryptoManager.fernet_engine(auth_key, salt_key)
        b_cs = engine.decrypt(encrypted_secret)
        return toolbox.ensure_string(b_cs)

    @staticmethod
    def encrypt_file(js: dict, fs: str, auth_key: str, salt_key: str):
        """
            Déchiffre les données d'un fichier et les retourne.

            Args:
            fs (str): Le chemin du fichier contenant les données chiffrées.

            Returns:
            str: Les données déchiffrées.
        """
        document = json.dumps(js)
        sensitive_data = toolbox.ensure_bytes(CryptoManager.encrypt_secret(document, auth_key, salt_key))

        with open(fs, 'wb') as file:
            file.write(sensitive_data)

    @staticmethod
    def decrypt_file(fs: str, auth_key: str, salt_key: str):
        """
            Déchiffre les données d'un fichier et les retourne.

            Args:
            fs (str): Le chemin du fichier contenant les données chiffrées.

            Returns:
            str: Les données déchiffrées.
        """
        try:
            with open(fs, 'rb') as file:
                encrypted_data = file.read()

            sensitive_data = CryptoManager.decrypt_secret(encrypted_data, auth_key, salt_key)
            return json.loads(sensitive_data)
        except Exception as ex:
            raise Exception('Erreur clé de chiffrage : {}'.format(ex))


# -*- coding: utf-8 -*-
# dreamtools-dreamgeeker/date_manager.py

"""
Module de Gestion des Dates
=============================

Fournit un ensemble de fonctions utilitaires pour manipuler les dates et heures en Python,
avec gestion des timezones, formats de dates, jours fériés et conversions diverses.

Constantes globales
-------------------
I_MON, I_TUES, I_WED, I_THU, I_FRI, I_SAT, I_SUN : Index numériques des jours de la semaine.

Timezones préconfigurées : tz_fr (Europe/Paris), tz_cay (America/Cayenne), tz_utc (UTC).

Formats de date disponibles :
- FR_FORMAT_DATE : '%d.%m.%Y'
- FR_FORMAT_DTIME : '%d-%m-%Y %H:%M:%S'
- FR_FORMAT_RSS_2 : '%a, %d %b %Y %H:%M:%S %z'
- FR_FORMAT_RSS_SHORT : '%a %d %b %Y'
- FR_FORMAT_RSS_ATOM : '%Y-%m-%dT%H:%M:%SZ'

Formats d'identifiants :
- FRM_ISO : format ISO 8601
- FRM_TIMESTAMP : timestamp UNIX

Toutes les fonctions respectent la timezone et peuvent travailler avec des objets `date`, `datetime`, ou `timestamp`.
"""

import locale
from datetime import datetime, timedelta, date

import pytz
from tzlocal import get_localzone_name

I_MON, I_TUES, I_WED, I_THU, I_FRI, I_SAT, I_SUN = 1, 2, 3, 4, 5, 6, 0

locale.setlocale(locale.LC_TIME, 'fr_FR.utf8')

nap_days_dc = {}

FR_FORMAT_DATE = '%d.%m.%Y'  # correction ici
FR_FORMAT_DTIME = '%d-%m-%Y %H:%M:%S'  # correction ici
FR_FORMAT_RSS_2 = '%a, %d %b %Y %H:%M:%S %z'  # correction ici
FR_FORMAT_RSS_SHORT = '%a %d %b %Y'  # correction ici
FR_FORMAT_RSS_ATOM = '%Y-%m-%dT%H:%M:%SZ'  # correction ici

FRM_ISO = 'iso'
FRM_TIMESTAMP = 'ts'


def get_timezone(name=None):
    if name:
        return pytz.timezone(name)
    return pytz.timezone(get_localzone_name())


tz_cay = get_timezone('America/Cayenne')
tz_fr = get_timezone('Europe/Paris')
tz_utc = pytz.UTC
tz_local = get_timezone()


def dte_to_dtime(dtime: date, ptz: pytz.timezone = tz_utc, force_midnight=False) -> datetime:
    """
    Convertit un objet `date` en `datetime` à minuit (00:00:00).

    Si l'objet est déjà un `datetime`, il est renvoyé tel quel.

    :param dtime: Date ou datetime à convertir
    :param ptz: Fuseau horaire à appliquer si conversion nécessaire (défaut : UTC)
    :param force_midnight: Si True, positionne systématiquement à minuit
    :return: Objet datetime timezone-aware

    :Exemple:
        >>> dte_to_dtime(date(2025, 1, 1))
        datetime.datetime(2025, 1, 1, 0, 0, tzinfo=<UTC>)
    """

    if isinstance(dtime, date) and not isinstance(dtime, datetime):
        dtime = datetime.combine(dtime, datetime.min.time())
        dtime = set_timezone(dtime, ptz=ptz)
    elif force_midnight:
        dtime = datetime.combine(dtime, datetime.min.time())

    return dtime


def get_midnight_dtime(dtime: date | None = None, ptz=tz_utc) -> datetime:
    """
    Renvoie un datetime positionné à minuit du jour indiqué (ou aujourd'hui).

    :param dtime: Date ou None (utilise aujourd'hui si None)
    :param ptz: Fuseau horaire à appliquer (défaut : UTC)
    :return: Datetime à 00:00:00
    """
    return dte_to_dtime(dtime or get_now_dtime(ptz), ptz, force_midnight=True)


def get_now_dtime(ptz=tz_utc) -> datetime:
    """
    Renvoie la date et l'heure actuelle avec timezone.

    :param ptz: Fuseau horaire à utiliser (défaut : UTC)
    :return: Datetime actuel timezone-aware
    """
    return datetime.now(ptz)


def get_now_str(ptz=tz_utc, fm=FR_FORMAT_DTIME) -> str:
    """
    Renvoie la date et l'heure actuelle formatée.

    :param ptz: Fuseau horaire à utiliser (défaut : UTC)
    :param fm: Format de la date (strptime)
    :return: Chaîne formatée

    :Exemple:
        >>> get_now_str()
        '12-07-2025 10:34:00'
    """

    return get_dtime_str(get_now_dtime(ptz), fm=fm)


def get_today_str(ptz=tz_utc, fm=FR_FORMAT_DATE) -> str:
    """
    Renvoie la date actuelle formatée sans heure.

    :param ptz: Fuseau horaire à utiliser (défaut : UTC)
    :param fm: Format de date (ex : '%d/%m/%Y')
    :return: Chaîne de date
    """
    return get_now_str(ptz, fm=fm)


def get_dtime_str(dtime: datetime | date | None = None, ptz: pytz.timezone = tz_utc,
                  fm: str = FRM_ISO) -> str | datetime:
    """
    Formate un datetime ou une date selon le format demandé.

    :param dtime: Date ou datetime à formater (None = maintenant)
    :param ptz: Fuseau horaire à appliquer (défaut : UTC)
    :param fm: Format de sortie ('iso' pour isoformat ou format strptime)
    :return: Chaîne formatée ou datetime
    """
    dtime = get_now_dtime(ptz) if dtime is None else dte_to_dtime(dtime, ptz)

    if fm == FRM_ISO:
        return dtime.isoformat()

    return dtime.strftime(fm)


def set_timezone(dtime: datetime, ptz=tz_utc) -> datetime:
    """
    Applique une timezone à un datetime naïf ou convertit entre timezones.

    :param dtime: Datetime à traiter
    :param ptz: Timezone cible (défaut : UTC)
    :return: Datetime
    """
    if dtime.tzinfo is None:
        return ptz.localize(dtime)
    elif dtime.tzinfo != ptz:
        return dtime.astimezone(ptz)
    else:
        return dtime


def dtime_to_local(dtime: datetime) -> datetime:
    """
    Convertit un datetime en datetime local .

    :param dtime: Datetime en UTC
    :return: Datetime dans la timezone locale
    """
    return set_timezone(dtime, tz_local)


def dtime_to_utc(dtime: datetime) -> datetime:
    """
    Convertit un datetime en datetime utc .

    :param dtime: Datetime en UTC
    :return: Datetime dans la timezone locale
    """
    return set_timezone(dtime, tz_local)


def dtime_to_fr(dtime: datetime) -> datetime:
    """
    Convertit un datetime en datetime france .

    :param dtime: Datetime en UTC
    :return: Datetime dans la timezone locale
    """
    return set_timezone(dtime, tz_local)


def get_dtime_ts(dtime: datetime | date | None = None, ptz: pytz.timezone = tz_utc) -> int:
    """
    Retourne le timestamp (secondes depuis Epoch) d'une date donnée.

    :param dtime: objet date, datetime ou None
    :param ptz: timezone de référence (défaut : UTC)
    :return: timestamp (int)
    """
    dtime = get_now_dtime(ptz) if dtime is None else dte_to_dtime(dtime, ptz)
    return int(dtime.timestamp())


def get_midnight_ts(dtime: datetime | date | None = None, ptz: pytz.timezone = tz_utc) -> int:
    """
    Retourne le timestamp correspondant à minuit du jour spécifié.

    :param dtime: Objet date ou datetime ou None
    :param ptz: timezone de référence
    :return: timestamp à 00:00 du jour
    """
    dtime = get_midnight_dtime(dtime, ptz)
    return int(dtime.timestamp())


def get_yesterday_ts(dtime: datetime | date | None = None, ptz: pytz.timezone = tz_utc) -> int:
    """
    Retourne le timestamp à minuit du jour précédent la date spécifiée.

    :param dtime: Date de référence ou None (par défaut : aujourd'hui)
    :param ptz: timezone de référence
    :return: timestamp à minuit du jour précédent
    """
    dtime = get_midnight_dtime(dtime, ptz)
    dtime = dtime_add_days(dtime, -1)

    return int(dtime.timestamp())


def ts_until_midnight(dtime: datetime | date | None = None, ptz: pytz.timezone = tz_utc) -> int:
    """
    Retourne le nombre de secondes jusqu'à minuit suivant la date donnée.

    :param dtime: date ou datetime (None = maintenant)
    :param ptz: timezone cible
    :return: nombre de secondes jusqu'à la prochaine minuit
    """
    dtime = get_midnight_dtime(dtime, ptz)
    today = get_midnight_ts(dtime, ptz)
    tonight = dtime_add_days(dtime)

    return int(tonight.timestamp() - today)


def utcnow_iso() -> str:
    """
    Retourne la date/heure actuelle UTC au format ISO 8601.

    :return str: date au format ISO
    """
    return get_dtime_str(ptz=tz_utc)


def utcnow_ts() -> int:
    """
    Retourne la date/heure actuelle UTC en timestamp (secondes).

    :return: timestamp UTC (int)
    """
    return get_dtime_ts(ptz=tz_utc)


def utcnow_str() -> str:
    """
    Retourne la date/heure actuelle UTC au format ISO.

    :return: chaîne de date ISO UTC (str)
    """
    return get_now_str(ptz=tz_utc, fm=FRM_ISO)


def set_dte(p_year, p_month, p_day) -> date:
    """
    Crée un objet `date` à partir de valeurs numériques.

    :param p_year: année (int)
    :param p_month: mois (int)
    :param p_day: jour (int)
    :return: objet date
    """
    return date(p_year, p_month, p_day)


def set_dtime(p_year, p_month, p_day, ptz: pytz.timezone = tz_utc) -> datetime:
    """
    Crée un objet `datetime` à partir d'une date numérique, positionnée à minuit.

    :param p_year: année (int)
    :param p_month: mois (int)
    :param p_day: jour (int)
    :param ptz: timezone (défaut UTC)
    :return: objet datetime timezone-aware à 00:00
    """
    dte = set_dte(p_year, p_month, p_day)
    return dte_to_dtime(dte, ptz)


def ts_to_dtime(ts) -> datetime:
    """
    Convertit un timestamp en objet `datetime`.

    :param ts: timestamp (int ou float)
    :return: datetime correspondant
    """
    return datetime.fromtimestamp(int(ts))  # => renvoie datetime


def ts_to_str(ts, ptz: pytz.timezone = tz_utc, fm=FR_FORMAT_DTIME):
    """
    Convertit un timestamp en chaîne formatée.

    :param ts: timestamp à convertir
    :param ptz: timezone pour conversion
    :param fm: format d'affichage (strftime)
    :return: chaîne de caractères représentant la date
    """
    dtime = ts_to_dtime(ts)
    return get_dtime_str(dtime, ptz, fm)


def str_to_dtime(s_dte: str, ptz=tz_utc, fm=FR_FORMAT_DTIME):
    """
    Convertit une chaîne formatée en objet `datetime` timezone-aware.

    :param s_dte: chaîne de date à convertir
    :param ptz: timezone cible
    :param fm: format de la chaîne (par défaut : '%d-%m-%Y %H:%M:%S')
    :return: objet datetime ou None si invalide

    :Exemple:
        >>> str_to_dtime('24-02-1976 16:45')
        datetime.datetime(1976, 2, 24, 16, 45, tzinfo=...)
    """
    dtime = datetime.strptime(s_dte, fm)
    return set_timezone(dtime, ptz)


def paques(y):
    """
    Calcule les dates des fêtes mobiles basées sur Pâques pour une année donnée.

    Retourne une liste avec :
    - dimanche de Pâques
    - lundi de Pâques (le lendemain)
    - jeudi de l'Ascension (39 jours après Pâques)
    - Pentecôte (49 jours après le lundi de Pâques)

    :param y: année de référence
    :return: liste de datetime timezone-aware (UTC)

    :Exemple:
        >>> paques(2025)
        [datetime.datetime(2025, 4, 20, 0, 0, tzinfo=...), datetime.datetime(2025, 4, 21, 0, 0, tzinfo=...), datetime.datetime(2025, 5, 29, 0, 0, tzinfo=...), datetime.datetime(2025, 6, 9, 0, 0, tzinfo=...)]
    """

    a = y // 100
    b = y % 100

    c = (3 * (a + 25)) // 4
    d = (3 * (a + 25)) % 4
    e = (8 * (a + 11)) // 25

    f = (5 * a + b) % 19
    g = (19 * f + c - e) % 30
    h = (f + 11 * g) // 319

    j = (60 * (5 - d) + b) // 4
    k = (60 * (5 - d) + b) % 4

    m = (2 * j - k - g + h) % 7
    n = (g - h + m + 114) // 31
    p = (g - h + m + 114) % 31

    jour = p + 1

    mois = n

    easter = set_dtime(y, mois, jour, tz_utc)
    easter_monday = dtime_add_days(easter, 1)
    ascension_day = dtime_add_days(easter, 39)
    pentecost = dtime_add_days(easter_monday, 49)

    return [easter, easter_monday, ascension_day, pentecost]


def jours_feries(y: int = None) -> list[datetime]:
    """
    Renvoie la liste des jours fériés pour l'année spécifiée.

    Inclut les jours fixes et les fêtes mobiles calculées à partir de Pâques.

    :param y: année (int), défaut = année actuelle
    :return: liste de datetime timezone-aware (UTC)

    :Exemple:
        >>> jours_feries(2025)
        [datetime.datetime(2025, 4, 20, 0, 0, tzinfo=...), ..., datetime.datetime(2025, 12, 25, 0, 0, tzinfo=...)]
    """
    list_jours_feries = nap_days_dc[f'{y}']
    if not list_jours_feries:
        y = get_now_dtime().year if y is None else int(y)

        list_jours_feries: list = paques(y)
        # jour de lan, fete du travail, jour victoire, fete national; assomption, tousain, armistice, noël
        l_dte = ((1, 1), (5, 1), (5, 8), (7, 14), (8, 15), (10, 1), (11, 1), (12, 25))

        for month, day in l_dte:
            list_jours_feries.append(set_dtime(y, month, day, tz_utc))
        nap_days_dc[f'{y}'] = list_jours_feries

    return list_jours_feries


def is_workday(dtime):
    """
    Détermine si une date est un jour ouvré (ni week-end ni jour férié).

    :param dtime: datetime à tester
    :return: True si jour ouvré, False sinon

    :Exemple:
        >>> is_workday(datetime(2025, 5, 1))
        False  # Fête du travail
        >>> is_workday(datetime(2025, 5, 2))
        True
    """
    feries = jours_feries(dtime.year)
    return not (dtime.weekday() in [I_SAT, I_SUN] or dtime in feries)


def dtime_add_days(dtime: datetime, nb: int = 1) -> datetime:
    """
    Ajoute un nombre de jours à un datetime.

    :param dtime: datetime de départ
    :param nb: nombre de jours à ajouter (positif ou négatif)
    :return: datetime ajusté

    :Exemple:
        >>> dtime_add_days(datetime(2025,1,1), 2)
        datetime.datetime(2025, 1, 3, 0, 0)
    """
    return dtime + timedelta(days=nb)


def dtime_add_hours(dtime: datetime, nb: int = 1) -> datetime:
    """
    Ajoute un nombre d'heures à un datetime.

    :param dtime: datetime de départ
    :param nb: nombre d'heures à ajouter
    :return: datetime ajusté
    """
    return dtime + timedelta(hours=nb)


def dtime_add_minutes(dtime: datetime, nb: int = 1) -> datetime:
    """
    Ajoute un nombre de minutes à un datetime.

    :param dtime: datetime de départ
    :param nb: nombre de minutes à ajouter
    :return: datetime ajusté
    """
    return dtime + timedelta(minutes=nb)


def dte_add_workday(dtime, nb):
    """
    Ajoute un nombre de jours ouvrés à une date.

    Incrémente la date en sautant week-ends et jours fériés.

    :param dtime: date de départ (datetime)
    :param nb: nombre de jours ouvrés à ajouter (positif)
    :return: datetime ajusté

    :Exemple:
        >>> dte_add_workday(datetime(2025,5,1), 3)
        datetime.datetime(2025, 5, 7, 0, 0)  # en sautant le 1er mai et le weekend
    """
    while nb > 0:
        dtime = dtime_add_days(dtime, 1)
        if is_workday(dtime):
            nb -= 1

    return dtime


def dtime_diff(dtea: datetime, dteb: datetime):
    """
    Calcule la différence en jours entre deux dates.

    :param dtea: première date
    :param dteb: deuxième date
    :return: différence absolue en jours (int)

    :Exemple:
        >>> dtime_diff(datetime(2025,1,1), datetime(2025,1,10))
        9
    """
    t = dteb - dtea
    return abs(t.days)


def dtime_compare(dtea: datetime, dteb: datetime):
    """
    Compare deux dates.

    :param dtea: première date
    :param dteb: deuxième date
    :return: 0 si égales, -1 si dteb < dtea, 1 sinon

    :Exemple:
        >>> dtime_compare(datetime(2025,1,1), datetime(2025,1,1))
        0
        >>> dtime_compare(datetime(2025,1,2), datetime(2025,1,1))
        -1
    """
    t = dteb - dtea

    if t.days == 0:
        return t.days
    elif t.days < 0:
        return -1
    else:
        return 1


def weeks_num(dtime: datetime | date | None = None):
    """
    Renvoie le numéro ISO de la semaine pour une date donnée (ou aujourd'hui par défaut).

    :param dtime: date ou datetime ou None
    :return: numéro de semaine (int)

    :Exemple:
        >>> weeks_num(datetime(2025,1,1))
        1
    """
    if dtime is None:
        dtime = get_now_dtime()
    else:
        dtime = dte_to_dtime(dtime)

    return dtime.isocalendar()[1]


def dtime_month_str(dtime):
    """
    Renvoie le mois et l'année d'une date sous forme de chaîne.

    :param dtime: datetime
    :return: chaîne formatée "Mois YYYY"

    :Exemple:
        >>> dtime_month_str(datetime(2025,1,15))
        'janvier 2025'
    """
    dtime = dte_to_dtime(dtime)
    return get_dtime_str(dtime, "%B %Y")


def month_to_fullmonth(month: int, year: None = None) -> str:
    """
    Renvoie le mois et l'année d'une date sous forme de chaîne.

    :param year:
    :param month:
    :return: chaîne formatée "Mois YYYY"

    :Exemple:
        >>> dtime_month_str(datetime(2025,1,15))
        'janvier 2025'
    """
    if year:
        year = get_now_dtime().year

    dte = set_dte(year, month, 1)
    return dtime_month_str(dte)


def dtime_rss_str(dtime=None, short=False):
    """
    Renvoie une date au format RSS.

    :param dtime: datetime ou None (aujourd'hui)
    :param short: bool, si True format court, sinon format complet
    :return: chaîne formatée

    :Exemple:
        >>> dtime_rss_str(datetime(2025,1,1), short=True)
        'Wed 01 Jan 2025'
    """
    dtime = dte_to_dtime(dtime)
    return get_dtime_str(dtime, fm=FR_FORMAT_RSS_SHORT) if short else get_dtime_str(dtime, fm=FR_FORMAT_RSS_2)


def day_in_hour(nb_day: int) -> int:
    """
    Convertit un nombre de jours en heures.

    :param nb_day: nombre de jours
    :return: nombre d'heures

    :Exemple:
        >>> day_in_hour(2)
        48
    """
    return nb_day * 24


def day_in_minutes(nb_day: int) -> int:
    """
    Convertit un nombre de jours en minutes.

    :param nb_day: nombre de jours
    :return: nombre de minutes

    :Exemple:
        >>> day_in_minutes(2)
        2880
    """
    return day_in_hour(nb_day) * 60


def day_in_sec(nb_day: int) -> int:
    """
    Convertit un nombre de jours en secondes.

    :param nb_day: nombre de jours
    :return: nombre de secondes

    :Exemple:
        >>> day_in_sec(1)
        86400
    """
    return day_in_minutes(nb_day) * 60


def current_year():
    # Test de la permission kozman_moderator
    return get_today_str(fm="%Y")


def day_in_msec(nb_day: int) -> int:
    """
    Convertit un nombre de jours en millisecondes.

    :param nb_day: nombre de jours
    :return: nombre de millisecondes

    :Exemple:
        >>> day_in_msec(1)
        86400000
    """
    return day_in_sec(nb_day) * 1000



import copy

_all_ = ['ImageManager']
# -*- coding: utf-8 -*-
# project/dreamtools-dreamgeeker/image_manager.py

import os
from io import BytesIO

from PIL import Image, ImageFile, ImageDraw
from PIL.ExifTags import TAGS
from PIL.TiffImagePlugin import ImageFileDirectory_v2

from . import file_manager
from .controller_manager import ControllerEngine
from .tracking_manager import TrackingManager

TYPE_IMG_JPEG = 'JPEG'
TYPE_IMG_PNG = 'PNG'
TYPE_IMG_GIF = 'GIF'
TYPE_IMG_WEBP = 'WEBP'

"""
Class CImagine
=============================

Class permettant le traitement d'une image png convertit en jpg avec prise en charge de la transparence (fond blanc)


pathfile : dreamtools-dreamgeeker/pyimaging

"""

MAX_SIZE = 640
MIN_SIZE = 200

class ImageManager(object):
    size_max:int
    size_thumb_max:int
    
    def __init__(self, src, dest, size_max=MAX_SIZE, size_thumb_max=MIN_SIZE):
        """
        Preparation image  pour traitement
        ==================================
        Les images sont convertit au format JPEG

        :param int size_max: taille maximum d'une image
        :param int size_thumb_max: taille miniature et minimum

        """
        self.extension = file_manager.file_extension(src).upper()

        if self.extension == "JPG":
            self.extension = TYPE_IMG_JPEG

        if self.extension not in [TYPE_IMG_JPEG, TYPE_IMG_PNG, TYPE_IMG_GIF, TYPE_IMG_WEBP]:
            raise ValueError("Format image non supporté")

        self.size_max = size_max
        self.size_thumb_max = size_thumb_max
        
        self.img = Image.open(src)
        self.exif = None

        self._size = self.img.size
        self.format = self.img.format

        self.file = file_manager.file_extension_less(dest)     # on s'assure de retirer l'extension

        self.resize(size_max)
    def __enter__(self):
        return self

    @property
    def _size(self):
        return self.w, self.h

    @_size.setter
    def _size(self, s):
        self.w, self.h = s

    def save_image_jpeg(self, file_name, img=None):
        if img is None:
            img = copy.deepcopy(self.img)  # copie profonde (réplique récursive)

        ImageManager.image_to_rgb(img)
        file_name = f'{file_name}.jpeg'

        if self.exif:
            img.save(file_name, TYPE_IMG_JPEG, exif=self.exif)
        else:
            img.save(file_name, TYPE_IMG_JPEG)

    def image_to_rgb(self):
        if self.img.mode != 'RGB':
            self.img = self.img.convert('RGB')

    def white_background(self):
        """
        Retourne l'image avec un fond blanc appliqué si l'image comporte de la transparence.
        :return: PIL.Image avec fond blanc
        """

        # Si l'image a un canal alpha (RGBA), on l’utilise comme masque
        if self.img.mode in ("RGBA", "LA"):
            bg = Image.new("RGB", self._size, (255, 255, 255))
            bg.paste(self.img, (0, 0), self.img)
            return bg
        return self.img

    def redraw_border(self, shape="rect", wc=False):
        """
            fs = '/home/dreamgeeker/Images/2151910934.jpg'
            fs_out = '/home/dreamgeeker/Images/tests.jpg'

            imager = ImageManager(fs, fs_out.format('circ'))
            imager.img = imager.redraw_border('circ')
            imager.save('png')
            imager = ImageManager(fs, fs_out.format('circ_border'))
            imager.img = imager.redraw_border('circ', True)
            imager.save('png')"""

        padding = 10 if wc else 0
        diameter = 2 * padding
        box = (self.img.width + diameter, self.img.height + diameter)

        if shape == "rect":
            bg = Image.new("RGB", box, (255, 255, 255))
            bg.paste(self.img, (padding, padding))
        else:  # shape == "circ":
            self.img.convert("RGBA")
            bg = Image.new("RGBA", box, (0, 0, 0, 0))
            if wc:
                draw = ImageDraw.Draw(bg)
                draw.ellipse((0, 0, box[0], box[1]), fill=(255, 255, 255, 255))

            # Masque circulaire
            mask = Image.new("L", self.img.size, 0)
            draw = ImageDraw.Draw(mask)
            draw.ellipse((0, 0, self.img.width, self.img.height), fill=255)

            pos_x = padding
            pos_y =padding

            # Coller l'image sur le cercle blanc au centre
            bg.paste(self.img, (pos_x, pos_y), mask)

        return bg

    def resize(self, size_max=MAX_SIZE, size_min=MIN_SIZE):
        """ Redimensionnement de l'image au format jpg

        :param size_min: taille minimum (carré)
        :param size_max:  taille maximum de l'image

        :return:
        """
        if self.h < size_min or self.w < size_min:
            raise Exception("Image trop petite")
        elif self.h < size_max or self.w < size_max:
            return

        if self.w >= self.h:
            h = self.h * self.size_max // self.w
            w = self.w
        else:
            w = self.w * self.size_max // self.h
            h = self.size_max

        self.img.resize((w, h))

    def generate_thumb(self, s=(MIN_SIZE,MIN_SIZE))->ImageFile:
        """ Thumb Image

        :param tuple[int, int] s: taille image, defaul (200, 200à

        """
        img = self.img.convert('L')
        img.thumbnail(s)
        return img

    def save(self, frm:str|None= None):
        """ Thumb Image
        :param frm:
        """
        frm = frm.lower() if frm else self.extension.lower()
        file_name = self.file + '.' + frm
        self.img.save(file_name, frm.upper())


    def save_thumb_image(self):
        """ Thumb Image
        """
        thumb = self.generate_thumb()
        self.save_image_jpeg(self.file + "_thumb", thumb)

    def protected(self, artist, description):
        """Ajoute un nom d'artist et le copyright d'une image"""

        _TAGS_r = dict(((v, k) for k, v in TAGS.items()))

        # Image File Directory
        ifd = ImageFileDirectory_v2()
        ifd[_TAGS_r["Artist"]] = artist
        ifd[_TAGS_r["Copyright"]] = 'Tous droits reserves'
        ifd[_TAGS_r["Description"]] = description

        out = BytesIO()
        ifd.save(out)

        self.exif = b"Exif\x00\x00" + out.getvalue()
        self.img.save(self.file, TYPE_IMG_JPEG, exif=self.exif)

    @staticmethod
    def directory_parsing(main_directory:str):
        """
        Redimensionne toutes les images contenu dans un répertoire donné + thumb
        :param main_directory:
        """
        for f in os.listdir(main_directory):
            f_path = os.path.join(main_directory, f)
            if os.path.isfile(f_path):
                imager = ImageManager(f_path, f_path)
                imager.save()
                imager.save_thumb_image()


    @staticmethod
    def treat_uploading(fs, fp):
        """
        Enregistrement d'une image uploaded (byte)

        :param file fs: filestream from flask request
        :param str fp: filepath d'enregistrement
        :return:
        """
        import uuid

        tmp = uuid.uuid1().int >> 64
        tmp_image = file_manager.path_build(ControllerEngine.TMP_DIR, str(tmp))

        try:
            TrackingManager.flag(f'[imgrecorder] Image temp recording : {tmp}')
            fs.save(tmp_image)

            TrackingManager.flag(f'[imgrecorder] Image treatment: {tmp} | {fp}')
            o = ImageManager(tmp_image, fp)
            o.save()

            TrackingManager.flag('[imgrecorder] Image thumbing')
            o.save_thumb_image()

            file_manager.remove_file(tmp_image)

            return True
        except Exception as e:
            TrackingManager.exception_tracking(e, '[imgrecorder] Aïe')
            return False


# -*- coding: utf-8 -*-
# config_manager.py
_all_ = ['ConfigController']
"""
Gestion fichiers de configurations (YAML)

pathfile : dreamtools-dreamgeeker/config_manager.py

Repertoires par défaut
----------------------


Class CFBases
-------------
"""
import yaml
from yaml import SafeLoader

from . import file_manager

class ConfigController:
    """
    cfg engine
    """

    @classmethod
    def loading(cls, p, ref=None, m='r'):
        """
        Récupération des parameters de configuration du fichier <p> section <r>

        :param str p: Fichier de configuration
        :param str ref: référence parameters à récupérer, optionnels
        :param str m: str|bytes par default
        :return: configuration | None

        """
        config = None

        if file_manager.path_exists(p):
            try:
                with open(p, mode=m) if 'b' in m else open(p, mode=m, encoding='utf-8') as cfg:
                    cfg = yaml.load(cfg, Loader=SafeLoader)
                    if type(cfg) == "dict":
                        cfg = dict(cfg)
                    elif type(cfg).__name__ == "list":
                        cfg = list(cfg)

                    config = cfg.get(ref) if ref else cfg
            except Exception as ex:
                print(f'[Chargement du fichier {p}:\n', ex)
        return config

    @classmethod
    def saving(cls, d, f, m="w"):
        """
        Enregistrement d'un fichier yaml
        ========================================

        :param dict(str, list(str)) d: données à enregistrer
        :param str f: nom du fichier
        :param str m: default (write): mode "w|a", optional
        :return:
        """
        file_manager.makedirs(file_manager.parent_directory(f))

        with open(f, m) if 'b' in m else open(f, mode=m, encoding='utf-8') as f_yml:
            yaml.dump(d, stream=f_yml, allow_unicode=True)

        return f



# -*- coding: utf-8 -*-
__all__ = [
    'config_manager', 'controller_manager', 'crypting_manager', 'date_manager',
    'exception_manager', 'file_manager', 'image_manager', 'mailing_manager',
    'toolbox', 'tracking_manager', 'validators_manager'
]
"""
Liste de fonctions standards réutilisables

.. author: Dreamgeeker
"""

__author__ = "Ketsia LENTIN"
__copyright__ = "Copyright 2021, CouleurWest IT"
__credits__ = "Ketsia LENTIN"
__license__ = "MIT"
__version__ = "2025.2.5.11"
__email__ = "saraswati@3p0.net"
__status__ = "Production"
__docformat__ = 'reStructuredText'

from . import config_manager
from . import controller_manager
from . import crypting_manager
from . import date_manager
from . import exception_manager
from . import file_manager
from . import image_manager
from . import mailing_manager
from . import toolbox
from . import tracking_manager
from . import validators_manager



from src.dreamtools.exception_manager import AccountException
from tests import fixation
from . import TrackingManager


def test_tracking_info(fixation):
    TrackingManager.info_tracking('# test 1 : Réussi', 'Test info tracking')


def test_tracking_warning(fixation):
    TrackingManager.info_tracking('# test 2 : Réussi', 'Test info tracking')


def test_tracking_exception(fixation):
    try:
        TrackingManager.flag('# test 3 : Réussi')
        raise Exception('Raised exception')
    except Exception as exception:
        TrackingManager.exception_tracking(exception, 'Test exception tracking')


def test_tracking_custom_exception(fixation):
    try:
        TrackingManager.flag('# test 4 : Réussi')
        raise AccountException()
    except Exception as exception:
        TrackingManager.exception_tracking(exception, 'Test custom exception tracking')
